from django import forms
from bootstrap_daterangepicker import widgets, fields
class FilterForm(forms.Form):
    date_range = fields.DateRangeField(
        input_formats=['MMMM D, YYYY', ""],
        widget=widgets.DateRangeWidget(
            format = 'MMMM D, YYYY',
            
        ),
        required=False
    )




# Generated by Django 3.1.7 on 2023-10-08 05:51

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('body', models.TextField()),
                ('provider', models.CharField(choices=[('kmi', 'Kmi'), ('jak', 'Jak'), ('geez', 'Geez'), ('africastalking', 'Africastalking'), ('none', 'None')], max_length=20)),
                ('sms_id', models.CharField(max_length=100, null=True, unique=True)),
                ('phone', models.CharField(max_length=20)),
                ('success', models.BooleanField(default=True)),
                ('sent_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



import requests
from requests.exceptions import ConnectTimeout
from ..exceptions import SmsException, FailedToSend, UnsupportedNumberType, InsufficientBalance, InvalidPhoneNumber, ServerError, SmsTimeOut
from ..sms_provider import SmsProvider


KMI_SMS_URL = 'http://api.kmicloud.com/sms/send/v1/notify'
KMI_BULK_URL = 'http://api.kmicloud.com/sms/send/v1/batch/marketing'

class KmiSms(SmsProvider):
    def __init__(self, access_key, secret_key, sender_id, callback_url) -> None:
        self.access_key = access_key
        self.secret_key = secret_key
        self.sender_id = sender_id
        self.callback_url = callback_url
        self.name = "kmi"

    def parse_phone(self, phone):
        return phone.replace("+", "00")
    
    def send_sms(self, phone, message):
        try:
            phone = self.parse_phone(phone)
            url = KMI_SMS_URL
            data = {
                "accessKey": self.access_key,
                "secretKey": self.secret_key,
                "from": self.sender_id,
                "to": phone ,
                "message": message,
                "callbackUrl":self.callback_url,
            }
            response = requests.post(url, json=data)
            json_response = response.json()

            if json_response['code'] == -106:
                raise InvalidPhoneNumber("Invalid phone number")
            if json_response['code'] == -104:
                raise UnsupportedNumberType("Unsupported phone number type")
            elif json_response['code'] == -30:
                raise InsufficientBalance("Insufficient balance")
            elif json_response['code'] >= 500:
                raise ServerError("Server error")
            elif json_response['code'] == 200 and json_response['success']:
                print("Success")
                return json_response["result"]
        except ConnectTimeout as error:
            raise SmsTimeOut(error)
        except SmsException as error:
            raise error
        except Exception as error:
            raise FailedToSend(error)
    
    def send_bulk_sms(self, phones, message):
        try:
            url = KMI_BULK_URL
            phones = [self.parse_phone(phone) for phone in phones]
            data = {
                "accessKey": self.access_key,
                "secretKey": self.secret_key,
                "from": self.sender_id,
                "to": phones,
                "message": message,
                "callbackUrl": self.callback_url
            }
            response = requests.post(url, json=data)
            json_response = response.json()

            if json_response['code'] == -106:
                raise InvalidPhoneNumber("Invalid phone number")
            if json_response['code'] == -104:
                raise UnsupportedNumberType("Unsupported phone number type")
            elif json_response['code'] == -30:
                raise InsufficientBalance("Insufficient balance")
            elif json_response['code'] >= 500:
                raise ServerError("Server error")
            elif json_response['code'] == 200 and json_response['success']:
                print("Success")
                return json_response
        except ConnectTimeout as error:
            raise SmsTimeOut(error)
        except SmsException as error:
            print(type(error))
            raise error
        except Exception as error:
            raise FailedToSend(error)
        


# Generated by Django 3.1.7 on 2023-10-08 18:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('smsapp', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='provider',
            field=models.CharField(blank=True, choices=[('kmi', 'Kmi'), ('jak', 'Jak'), ('geez', 'Geez'), ('africastalking', 'Africastalking')], max_length=20, null=True),
        ),
    ]



import requests
from requests.exceptions import ConnectTimeout
from ..exceptions import SmsException, FailedToSend, SmsTimeOut
from ..sms_provider import SmsProvider

GEEZ_URL = 'https://api.geezsms.com/api/v1/sms/send'

class GeezSms(SmsProvider):
    def __init__(self, token) -> None:
        self.token = token
        self.name = "geez"

    def parse_phone(self, phone):
        return phone.replace('+', '', 1)
    
    def send_sms(self, phone, message):
        try:
            url = GEEZ_URL
            data = {
                'token': self.token,
                'msg': message,
                'phone': phone,
                'shortcode_id': "Guzo"
            }
            response = requests.post(url, data=data)

            print(response.json())
            if response.status_code == 200 or str(response.content).startswith('Successfully'):
                return response
            else:
                raise FailedToSend(f"{self.name} failed to send!")
        except ConnectTimeout as error:
            raise SmsTimeOut(error)
        except FailedToSend as error:
            raise error
        except Exception as error:
            raise SmsException(error)

    def send_bulk_sms(self, phones, message):
        for phone in phones:
            self.send_sms(phone, message)
            


class SmsException(Exception):
    pass

class AllProvidersFailed(SmsException):
    pass

class InvalidPhoneNumber(SmsException):
    pass

class InsufficientBalance(SmsException):
    pass

class UnsupportedNumberType(SmsException):
    pass

class InsufficientBalance(SmsException):
    pass

class ServerError(SmsException):
    pass

class SmsConnectionError(SmsException):
    pass

class FailedToSend(SmsException):
    pass

class SmsTimeOut(SmsException):
    pass


import africastalking
from ..exceptions import SmsException, FailedToSend, UnsupportedNumberType, InsufficientBalance, InvalidPhoneNumber, ServerError
from ..sms_provider import SmsProvider


class AfricasTalkingSms(SmsProvider):
    def __init__(self, username, api_key, sender_id) -> None:
        self.username = username
        self.api_key = api_key
        self.sender_id = sender_id
        self.name = 'africastalking'
        


    def parse_phone(self, phone):
        return [phone]

    def send_sms(self, phone, message):            
        try:            
            africastalking.initialize(self.username, self.api_key)
            sms = africastalking.SMS
            phone = self.parse_phone(phone)
            response = sms.send(message, phone, sender_id=self.sender_id)           
            recipients = response["SMSMessageData"]["Recipients"]
            
            
            if recipients[0]["statusCode"] == 403:
                raise InvalidPhoneNumber("Invalid phone number")
            if recipients[0]["statusCode"] == 404:
                raise UnsupportedNumberType("Unsuppoted phone number type")
            elif recipients[0]["statusCode"] == 405:
                raise InsufficientBalance("Insufficient balance")
            elif recipients[0]["statusCode"] >= 500:
                raise ServerError("Server Error")
            elif recipients[0]["status"] == "Success" and recipients[0]["statusCode"] == 101:
                recipients[0]["smsId"] = recipients[0]["messageId"]
                return recipients[0]

        except SmsException as error:
            raise error
        except Exception as error:
            print(f"Error: {error}")
            raise FailedToSend(error)
    
    def send_bulk_sms(self, phones, message):
        def on_finish(error, data):
            if error:
                raise error
            recipients = data["SMSMessageData"]["Recipients"]
            print(recipients[-1])
            assert len(recipients) <= len(phones)

        response = self.sms.send(message, phones, sender_id=self.sender_id, enqueue=True, callback=on_finish)
        
        recipients = response["SMSMessageData"]["Recipients"]
        if recipients[0]["status"] == "Success":
            return response
        raise FailedToSend(f"{self.name} Failed to send!")


from django.shortcuts import render, redirect

from django.http import JsonResponse
from . models import Message
from . forms import FilterForm
from django.db.models import Count, Q
from django.db.models.functions import TruncDate, TruncMonth
from django.utils import timezone
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.utils.timezone import make_aware
import datetime
from datetime import datetime as dt

from . utils import data_perday, data_permonth

# Create your views here.
def index(request):
    sms_id = request.GET.get("smsId")
    phone = request.GET.get("to")
    response = request.GET.get('sendResult')
    dr = request.GET.get("dr")

    if dr == "000":
        message = Message.objects.get(sms_id=sms_id, phone=phone)
        message.success = True
        message.save()

    return JsonResponse({"message": response, "code": dr})

def chart_view(request):
    #Bar Chart Data for total number of messages per day
    form = FilterForm(request.GET)
    
    end_date = timezone.now()
    start_date = end_date - datetime.timedelta(days=7)
    
    if "date_range" in request.GET:
        date_range = request.GET["date_range"]
        start, end = date_range.split('-')
        start_date = make_aware(dt.strptime(start, '%B %d, %Y '))
        # Upto the time the request came
        hour = timezone.now().hour
        min = timezone.now().minute + 1
        end_date = make_aware(dt.strptime(end, ' %B %d, %Y')) + datetime.timedelta(hours=hour, minutes=min)
    
    
    # If the date range is greater than a month group by month 
    trunc = TruncDate 
    diff = end_date - start_date
    if diff.days > 31:
        trunc = TruncMonth
    
    sms_data = Message.objects.filter(sent_at__range=(start_date, end_date))\
        .annotate(date=trunc('sent_at'))\
            .values('date')\
                .annotate(success_count=Count('id', filter=Q(success=True)), failed_count=Count('id', filter=Q(success=False)))\
                    .order_by('date')
    
    x_bardata, y_bardata = data_perday(sms_data, start_date, end_date)
    if diff.days > 31:
        x_bardata, y_bardata = data_permonth(sms_data, start_date, end_date)

    # Pie Chart Data
    providers_query = Message.objects.filter(success=True)\
        .values('provider').annotate(count = Count('id'))
    
    providers = list(map(str, Message.Providers))
    
    providers_data = []
    for provider in providers:
        item = providers_query.filter(provider = provider)
        count = item[0]['count'] if item else 0
        providers_data.append(count) 
    

    # Table Data
    
    all_message = Message.objects.all().order_by("-sent_at")
    
    paginator = Paginator(all_message, 3)
    table_data = []
    page = int(request.GET.get("page", 1))
    
    try:
        for i in range(page):
            page_data = paginator.page(i+1)
            table_data += page_data.object_list
    except PageNotAnInteger:
        table_data = paginator.page(1) 
    except EmptyPage:
        for p in paginator.page_range[paginator.num_pages]:
            page_data = paginator.page(p+1)
            table_data += page_data.object_list
    
    context = {
        "form": form,
        "providers": providers,
        "providers_data": providers_data, 
        "y_bardata": y_bardata,
        "total_success_sms": sum(list(map(lambda x: x["success_count"], y_bardata))),
        "total_failed_sms": sum(list(map(lambda x: x["failed_count"], y_bardata))),
        "total_sms": sum(list(map(lambda x: x["failed_count"]+x["success_count"], y_bardata))),
        "x_bardata": x_bardata,
        "page_data": page_data,
        "table_data": table_data,
    }
    
    return render(request, 'chart/index.html', context)
    


from django.db import models
# import sms.utils as utils

class Message(models.Model):

    class Providers(models.TextChoices):
        KMI = 'kmi'
        JAK = 'jak'
        GEEZ = 'geez'
        AFRICASTALKING = 'africastalking'
        

    body = models.TextField()
    provider = models.CharField(max_length=20, choices=Providers.choices, null=True, blank=True)
    sms_id = models.CharField(max_length=100, unique=True, null=True, blank=True)
    phone = models.CharField(max_length=20)
    success = models.BooleanField(default=False,)
    sent_at = models.DateTimeField(auto_now_add=True)

    def __str__(self) -> str:
        return (self.provider if self.provider else "FAILED") + ": " + self.phone + " " + self.body







# Generated by Django 3.1.7 on 2023-10-12 04:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('smsapp', '0004_auto_20231009_0504'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='success',
            field=models.BooleanField(default=False),
        ),
    ]



from abc import ABC, abstractmethod

class SmsProvider(ABC):
    @abstractmethod
    def send_sms(self, *args):
        pass

    @abstractmethod
    def send_bulk_sms(self, *args):
        pass

    @abstractmethod
    def parse_phone(self, phone):
        pass



import requests
from ..exceptions import SmsException, FailedToSend, SmsTimeOut
from ..sms_provider import SmsProvider

from requests.exceptions import ConnectTimeout

JAK_URL = 'http://197.156.70.196:9095/api/send_sms'

class JakSms(SmsProvider):
    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.name = "jak"

    def parse_phone(self, phone):
        return phone.replace("+", "", 1)
    
    def send_sms(self, phone, message):
        try:
            url = JAK_URL
            phone = self.parse_phone(phone)
            
            response = requests.post(url, json={
                "username": self.username,
                "password": self.password,
                "phone": phone,
                "msg": message
            })
            print(response.json())

            if response.status_code == 200 or str(response.content).startswith('Successfully'):
                return response
            else:
                raise FailedToSend(f"{self.name} failed to send!")
        except ConnectTimeout as error:
            raise SmsTimeOut(error)
        except FailedToSend as error:
            raise error
        except Exception as error:
            raise SmsException(error)
        
    def send_bulk_sms(self, phones, message):
        for phone in phones:
            self.send_sms(phone, message)



# from sms.settings import AFRICAS_USERNAME, AFRICAS_API_KEY, SENDER_ID, KMI_ACCESS_KEY, KMI_SECRET_KEY, GEEZ_TOKEN, JAK_USERNAME, JAK_PASSWORD
# from .providers.kmi_sms import KmiSms
# from .providers.africastalking_sms import AfricasTalkingSms
# from .providers.jak_sms import JakSms
# from .providers.geez_sms import GeezSms
from .exceptions import SmsException, FailedToSend, AllProvidersFailed
from smsapp.models import Message

import datetime
from dateutil.relativedelta import relativedelta
from django.utils.timezone import make_aware

# providers = [
#     KmiSms(access_key=KMI_ACCESS_KEY, secret_key=KMI_SECRET_KEY, sender_id=SENDER_ID, callback_url="https://ad04-196-188-55-159.ngrok-free.app/sms_callback"),
#     AfricasTalkingSms(username=AFRICAS_USERNAME, api_key=AFRICAS_API_KEY, sender_id=SENDER_ID),
#     JakSms(username=JAK_USERNAME, password=JAK_PASSWORD),
#     GeezSms(GEEZ_TOKEN),
# ]


def send_sms(phone, message, providers):
    for provider in providers:
        try:
            print(f"Sending to {phone} with Provider {provider.name}")
            response = provider.send_sms(phone, message)
        
            return (response, provider.name)
        except FailedToSend as error:            
            print(error)
            continue       
        except SmsException as error:
            print(f"{provider.name} failed to send due to {error}")
            continue
        except Exception as error:
            print(error)
            continue
    
    raise AllProvidersFailed("All providers have failed to send.")



def send_and_save_sms(phone, smsbody):
    message = Message(phone=phone, body=smsbody,)
    try:
        response, provider = send_sms(phone, smsbody)
        print(response)
        message.provider = provider
        message.sms_id = response['smsId']
    except AllProvidersFailed:
        raise
    finally:
        message.save()


def resend(since, to):
    failed_messages = Message.objects.filter(success = False, sent_at__range=(since, to))
    sms_count = int(failed_messages.count())
    failed_again = 0

    for message in failed_messages:
        phone = message.phone
        body = message.body
        try:
            response, provider = send_sms(phone, body)
            message.provider = provider
            message.sms_id = response['sms_id']
            message.save()
        except AllProvidersFailed:
            failed_again += 1
            continue

    return f"Successfully resend {sms_count - failed_again} out of {sms_count}"

def send_bulk_sms(phones, message, providers):
    for provider in providers:
        try:
            print(f"Sending using Provider {provider.name}")

            response = provider.send_bulk_sms(phones, message)

            return response
        except AssertionError:
            print(f"Assertion error on provider {provider.name}")
            continue
        except FailedToSend as error:
            print(error)
            continue
        except SmsException as error:
            print(f"{provider.name} failed to send due to {error}")
            continue
        
    raise AllProvidersFailed("All providers have failed to send.")

DAY_OF_WEEK = ["Mon", "Tus", "Wed", "Thu", "Fri", "Sat", "Sun"]
MONTHS = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]


def data_perday(sms_data, start, end):
    days = []
    totalsmsperday = []
    currentDay = start
    day = 0
    diff = end - start

    while currentDay < end:
        currentDay = start + datetime.timedelta(days=day)
        currentDate = currentDay.date()
        sms = sms_data.filter(date=currentDate)
        
        smsperday = sms[0] if sms else {'success_count': 0, 'failed_count': 0}

        totalsmsperday.append(smsperday)
        days.append(date_format(currentDay, diff.days))
        day += 1
    return (days, totalsmsperday)

def data_permonth(sms_data, start, end):
    months = []
    totalsmspermonth = []
    currentDate = make_aware(datetime.datetime(start.year, start.month, 1))
    month = 0
    while currentDate <= end:
        currentDate = start + relativedelta(months=month)

        sms = sms_data.filter(date=currentDate)
        
        smspermonth = sms[0] if sms else {'success_count': 0, 'failed_count': 0}

        totalsmspermonth.append(smspermonth)
        months.append(MONTHS[currentDate.month - 1])
        month += 1
    return (months, totalsmspermonth)


def date_format(date, diff):
    if diff < 15:
        return DAY_OF_WEEK[date.weekday()]
    else:
        return f"{MONTHS[date.month - 1]}, {date.day}"
    
